﻿using Android.App;
using Android.Widget;
using Android.OS;
using System.IO;
using Syncfusion.SfPdfViewer.Android;
using System;
using Android.Views;
using Android.Views.InputMethods;
using Android.Content;
using System.Diagnostics;
using System.Linq;

namespace PdfViewerDemo
{
    [Activity(Label = "PdfViewerDemo", MainLauncher = true, Icon = "@drawable/icon" , ConfigurationChanges = Android.Content.PM.ConfigChanges.Orientation | Android.Content.PM.ConfigChanges.ScreenSize)]
    public class MainActivity : Activity
    {
        SfPdfViewer pdfViewerControl;
        EditText pageNumberEntry;
        TextView pageCountText;
        ImageButton pageDownButton;
        ImageButton pageUpButton;
        LinearLayout mainView;
        GridLayout toolBarGrid;
        GridLayout searchBarGrid;
        EditText searchView;
        ImageButton searchButton;
        ImageButton backButton;
        Stopwatch stopwatch;
        ImageButton clearSearchButton;
        Context pdfViewerContext;
        ImageButton selectionButton;
        string backupDocumentName = string.Empty;
        ImageButton searchNextButton, searchPrevButton;

        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);
            // Set our view from the "main" layout resource
            SetContentView(Resource.Layout.Main);
            mainView = FindViewById<LinearLayout>(Resource.Id.mainView);
            pdfViewerContext = mainView.Context;
            pageNumberEntry = (EditText)FindViewById(Resource.Id.pagenumberentry);
            pageNumberEntry.KeyPress += PageNumberEntry_KeyPress;
            pageCountText = (TextView)FindViewById(Resource.Id.pagecounttext);
            pageDownButton = (ImageButton)FindViewById(Resource.Id.pagedownbutton);
            pageDownButton.Click += PageDownButton_Click;
            pageUpButton = (ImageButton)FindViewById(Resource.Id.pageupbutton);
            pageUpButton.Click += PageUpButton_Click;
            searchButton = FindViewById<ImageButton>(Resource.Id.searchButton);
            searchButton.Click += SearchButton_Click;
            pdfViewerControl = FindViewById<SfPdfViewer>(Resource.Id.pdfviewercontrol);
            pdfViewerControl.DocumentLoaded += PdfViewerControl_DocumentLoaded;
            pdfViewerControl.PageChanged += PdfViewerControl_PageChanged;
            backupDocumentName = "GIS Succinctly";
            Stream docStream = Assets.Open(backupDocumentName+".pdf");

            dialog = new ProgressDialog(this);
            stopwatch = new Stopwatch();

            toolBarGrid = FindViewById<GridLayout>(Resource.Id.toolbarGrid);
            searchBarGrid = FindViewById<GridLayout>(Resource.Id.searchGrid);
            searchBarGrid.Visibility = ViewStates.Invisible;

            backButton = FindViewById<ImageButton>(Resource.Id.backButton);
            backButton.Click += BackButton_Click;
            searchView = FindViewById<EditText>(Resource.Id.search);
            searchView.SetHintTextColor(Android.Graphics.Color.Rgb(189, 189, 189));
            searchView.Hint = "Search text";
            searchView.FocusableInTouchMode = true;
            searchView.TextSize = 18;
            searchView.SetTextColor(Android.Graphics.Color.Rgb(103, 103, 103));
            searchView.TextAlignment = TextAlignment.Center;
            searchView.KeyPress += SearchView_KeyPress;
            searchView.TextChanged += SearchView_TextChanged;
            selectionButton = FindViewById<ImageButton>(Resource.Id.selectDocumentButton);
            selectionButton.Click += SelectionButton_Click;
            pdfViewerControl.TextMatchFound += PdfViewerControl_TextMatchFound;
            clearSearchButton = FindViewById<ImageButton>(Resource.Id.clearSearchResult);
            clearSearchButton.Click += ClearSearchButton_Click;
            pdfViewerControl.SearchInitiated += PdfViewerControl_SearchInitiated;
            pdfViewerControl.SearchCompleted += PdfViewerControl_SearchProgressCompleted;
            searchNextButton = FindViewById<ImageButton>(Resource.Id.searchNext);
            searchPrevButton = FindViewById<ImageButton>(Resource.Id.searchPrev);
            searchNextButton.Click += SearchNextButton_Click;
            searchPrevButton.Click += SearchPrevButton_Click;
            pdfViewerControl.LoadDocument(docStream);


        }
        private void SearchPrevButton_Click(object sender, EventArgs e)
        {
            if (searchView.Text != string.Empty)
                pdfViewerControl.SearchPrevious(searchView.Text);
        }

        private void SearchNextButton_Click(object sender, EventArgs e)
        {
            if (searchView.Text != string.Empty)
                pdfViewerControl.SearchNext();
        }

        #region DocumentSelectionMenu
        private void SelectionButton_Click(object sender, EventArgs e)
        {
            PopupMenu popup = new PopupMenu(pdfViewerContext, selectionButton);
            popup.Inflate(Resource.Drawable.popup_menu_pdfViewer);
            popup.MenuItemClick += Popup_MenuItemClick;
            popup.Show();
        }

        private void Popup_MenuItemClick(object sender, PopupMenu.MenuItemClickEventArgs e)
        {
            string documentName = e.Item.TitleFormatted.ToString();
            if (documentName != backupDocumentName)
            {
                Stream docStream = Assets.Open(documentName + ".pdf");
                pdfViewerControl.LoadDocument(docStream);
                backupDocumentName = documentName;
            }
        }
        #endregion

        #region TextSearch
        private void SearchView_TextChanged(object sender, Android.Text.TextChangedEventArgs e)
        {
            if (searchView.Text != string.Empty)
            {
                clearSearchButton.Visibility = ViewStates.Visible;
            }
            else
            {
                clearSearchButton.Visibility = ViewStates.Invisible;
            }
        }

        private void ClearSearchButton_Click(object sender, EventArgs e)
        {
            pdfViewerControl.CancelSearch();
            searchView.Text = "";
            isTextFound = false;
            clearSearchButton.Visibility = ViewStates.Invisible;
            InputMethodManager inputMethodManager = (InputMethodManager)mainView.Context.GetSystemService(Context.InputMethodService);
            inputMethodManager.ShowSoftInput(searchView, ShowFlags.Implicit);
        }

        private void SearchView_KeyPress(object sender, View.KeyEventArgs e)
        {
            var editText = sender as EditText;
            if (e.KeyCode == Keycode.Enter)
            {
                if (!string.IsNullOrWhiteSpace(editText.Text) && !string.IsNullOrEmpty(editText.Text))
                {
                    stopwatch.Start();
                    searchText = editText.Text;
                    pdfViewerControl.SearchText(searchText);
                }
                InputMethodManager inputMethodManager = (InputMethodManager)this.ApplicationContext.GetSystemService(Context.InputMethodService);
                inputMethodManager.HideSoftInputFromWindow(mainView.WindowToken, HideSoftInputFlags.None);
            }
            if (e.KeyCode == Keycode.Del)
            {
                editText.Text = editText.Text.Remove(editText.Length() - 1, 1);
                editText.SetSelection(editText.Text.Length);
            }
        }

        private void BackButton_Click(object sender, EventArgs e)
        {
            if (searchBarGrid.Visibility == ViewStates.Visible)
            {
                searchBarGrid.Visibility = ViewStates.Invisible;
                toolBarGrid.Visibility = ViewStates.Visible;
                pdfViewerControl.CancelSearch();
                searchView.Text = "";
                isTextFound = false;
                clearSearchButton.Visibility = ViewStates.Invisible;
                InputMethodManager inputMethodManager = (InputMethodManager)mainView.Context.GetSystemService(Context.InputMethodService);
                inputMethodManager.HideSoftInputFromWindow(mainView.WindowToken, HideSoftInputFlags.None);
            }
        }

        private void SearchButton_Click(object sender, EventArgs e)
        {
            if (toolBarGrid.Visibility == ViewStates.Visible)
            {
                toolBarGrid.Visibility = ViewStates.Invisible;
                searchBarGrid.Visibility = ViewStates.Visible;
                searchView.RequestFocus();
                clearSearchButton.Visibility = ViewStates.Invisible;
                InputMethodManager inputMethodManager = (InputMethodManager)mainView.Context.GetSystemService(Context.InputMethodService);
                inputMethodManager.ShowSoftInput(searchView, ShowFlags.Implicit);
            }
        }

        string searchText = string.Empty;
        ProgressDialog dialog;
        bool isTextFound;

        private void PdfViewerControl_TextMatchFound(object sender, TextMatchFoundEventArgs args)
        {
            stopwatch.Stop();
            timerStarted = false;
            if (!stopwatch.IsRunning)
            {
                if (dialog != null)
                {
                    dialog.Hide();
                }
            }
        }

        private void PdfViewerControl_SearchProgressCompleted(object sender, TextSearchCompletedEventArgs args)
        {
            stopwatch.Stop();
            timerStarted = false;
            if (!stopwatch.IsRunning)
            {
                if (dialog != null)
                {
                    dialog.Hide();
                    if (args.NoMatchFound)
                    {
                        var toast = Toast.MakeText(pdfViewerContext, "No matches found...", ToastLength.Short);
                        toast.SetGravity(GravityFlags.Center, 0, 0);
                        toast.Show();
                    }
                    else if (args.NoMoreOccurrence)
                    {
                        var toast = Toast.MakeText(pdfViewerContext, "No more occurrences...", ToastLength.Short);
                        toast.SetGravity(GravityFlags.Center, 0, 0);
                        toast.Show();
                    }
                    else if (!isTextFound)
                    {
                        isTextFound = true;
                    }
                }
            }
        }

        bool timerStarted = false;
        private void PdfViewerControl_SearchInitiated(object sender, TextSearchInitiatedEventArgs args)
        {
            if (!stopwatch.IsRunning)
                stopwatch.Start();
            else
            {
                stopwatch.Stop();
            }
            if (stopwatch.ElapsedMilliseconds > 2000 && !timerStarted)
            {
                timerStarted = true;
                dialog.SetMessage("Search Progress...");
                dialog.Show();
            }
        }

        #endregion


        private void PageUpButton_Click(object sender, EventArgs e)
        {
            pdfViewerControl.GoToPreviousPage();
        }

        private void PageDownButton_Click(object sender, EventArgs e)
        {
            pdfViewerControl.GoToNextPage();
        }

        private void PdfViewerControl_PageChanged(object sender, PageChangedEventArgs args)
        {
            pageNumberEntry.Text = args.PageNumber.ToString();
        }

        private void PdfViewerControl_DocumentLoaded(object sender, EventArgs args)
        {
            pageNumberEntry.Text = pdfViewerControl.PageNumber.ToString();
            pageCountText.Text = pdfViewerControl.PageCount.ToString();
        }

        private void PageNumberEntry_KeyPress(object sender, View.KeyEventArgs e)
        {
            e.Handled = false;
            if (e.Event.Action == KeyEventActions.Down && e.KeyCode == Keycode.Enter)
            {
                int pageNumber = 0;
                if (int.TryParse((pageNumberEntry.Text), out pageNumber))
                {
                    if ((pageNumber > 0) && (pageNumber <= pdfViewerControl.PageCount))
                        pdfViewerControl.GoToPage(pageNumber);
                    else
                    {
                        DisplayAlertDialog();
                        pageNumberEntry.Text = pdfViewerControl.PageNumber.ToString();
                    }
                }
                pageNumberEntry.ClearFocus();
                InputMethodManager inputMethodManager = (InputMethodManager)this.GetSystemService(Context.InputMethodService);
                inputMethodManager.HideSoftInputFromWindow(mainView.WindowToken, HideSoftInputFlags.None);
            }
        }

        void DisplayAlertDialog()
        {
            AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);
            alertDialog.SetTitle("Error");
            alertDialog.SetMessage("Please enter the valid page number");
            alertDialog.SetPositiveButton("OK", (senderAlert, args) => { });
            Dialog dialog = alertDialog.Create();
            dialog.Show();
        }

    }
}

